/*  
  multiplex pattern for Titchaino with switch
    by ikkei 2014.07.09
*/
 
#include <arduino.h>   
#include <MsTimer2.h>
#include <ikkeiplexing.h>
#include <EEPROM.h>
#include "iFont.h"

#include "complex_pattern.h"
#include "heartbeat.h"
#include "random_fade.h"
#include "wave_animation.h"
#include "scroll_text.h"
#include "bounce_ball.h"
#include "dragon.h"

static ikkeiplexing iLedSign;
static iFont iFont;

#define FRAME  0x01
#define TENMS  0x02

static byte icount;
static byte period;


static byte state;
static byte mode_num;
void mode_setup(byte num){
  icount++;
  //if (icount >= period){
  if (icount >= 20){
    icount = 0;
    state = num;
    switch(num){
      case 1: // space invaders comlex pattern
        complex_pattern_init();
        break;
      case 2: // heart beat
        heartbeat_init();
        break;
      case 3: // random fade
        random_fade_init();
        break;
      case 4: // wave
        wave_animation_init();
        break;
      case 5: // scroll text "Titchaino"
        scroll_text_init();
        break;
      case 6: // scroll text "Titchaino"
        bounce_ball_init();
        break;
      case 7: // scroll text "Titchaino"
        dragon_init();
        break;
    }
  }
}

void display_number(byte num){
  iLedSign.Clear();
  byte d = iFont.Width('0' + num);
  for (byte i= 0; i < d; i++){
    iLedSign.Vwrite(i + 7 - d/2, iFont.Data('0' + num,i));
  }
  period = 20;
  icount = 0;
  state = 0;
}

void setup() {
  iLedSign.Init(TITCHAINO, SINGLE_BUFFER);
  mode_num = EEPROM.read(0);
  if (mode_num > 7 || mode_num == 0){
    mode_num = 1;
    EEPROM.write(0, mode_num);
  }
  mode_setup(mode_num);
}


void loop() {
  if ((iLedSign.GetPeriod() & FRAME) != 0){
    uint8_t event = iLedSign.GetSW(1);
    if ((event & 0xC0) != 0){
      mode_num++;
      if (mode_num > 7 || mode_num == 0){
        mode_num = 1;
      }
      EEPROM.write(0, mode_num);
      display_number(mode_num);
    }
    switch (state){
      case 0:
        mode_setup(mode_num);
        break;
      case 1:
        invader();
        break;
      case 2:
        heartbeat();
        break;
      case 3:
        random_fade();
        break;
      case 4:
        wave_animation();
        break;
      case 5:
        scroll_text();
        break;
      case 6:
        bounce_ball();
        break;
      case 7:
        dragon();
        break;
    }
  }
}

